#---
# Excerpted from "Programming Phoenix LiveView",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material,
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose.
# Visit https://pragprog.com/titles/liveview for more book information.
#---
defmodule PentoWeb.GameLive.Board do
  use PentoWeb, :live_component

  alias Pento.Game.{Board, Pentomino}
  import PentoWeb.GameLive.{Colors, Component}


  def update(%{puzzle: puzzle, id: id}, socket) do
    {:ok,
      socket
      |> assign_params(id, puzzle)
      |> assign_board()
      |> assign_shapes()
    }
  end

  def assign_params(socket, id, puzzle) do
    assign(socket, id: id, puzzle: puzzle)
  end

  def assign_board(%{assigns: %{puzzle: puzzle}} = socket) do
    active = Pentomino.new(name: :p, location: {7, 2})
    completed = [
        Pentomino.new(name: :u, rotation: 270, location: {1, 2}),
        Pentomino.new(name: :v, rotation: 90, location: {4, 2})
      ]
    _puzzles = Board.puzzles() # atom must exist!

    board =
      puzzle
      |> String.to_existing_atom
      |> Board.new
      |> Map.put(:completed_pentos, completed)
      |> Map.put(:active_pento, active)

    assign(socket, board: board)
  end



  def assign_shapes(%{assigns: %{board: board}} = socket) do
    shapes = Board.to_shapes(board)
    assign(socket, shapes: shapes)
  end

  def render(assigns) do
    ~H"""
    <div id={ @id } phx-window-keydown="key" phx-target={ @myself }>
      <.canvas view_box="0 0 200 70">
        <%= for shape <- @shapes do %>
          <.shape
            points={ shape.points }
            fill= { color(shape.color, Board.active?(@board, shape.name), false) }
            name={ shape.name } />
        <% end %>
      </.canvas>
      <hr/>
      <.palette
        shape_names={@board.palette}
        completed_shape_names=
          {Enum.map(@board.completed_pentos, & &1.name)} />
    </div>
    """
  end
end
